@@ -38,9 +38,7 @@ from ..url import PrivacyMode
 from ..common import NotifyImageSize
 from ..common import NotifyFormat
 from ..common import NotifyType
-from ..utils import parse_list
-from ..utils import parse_bool
-from ..utils import validate_regex
+from ..utils.parse import parse_list, parse_bool, validate_regex
 from ..locale import gettext_lazy as _
 from ..attachment.base import AttachBase
 
@@ -336,6 +334,18 @@ class NotifyMastodon(NotifyBase):
 
         return
 
+    @property
+    def url_identifier(self):
+        """
+        Returns all of the identifiers that make this URL unique from
+        another simliar one. Targets or end points should never be identified
+        here.
+        """
+        return (
+            self.secure_protocol[0], self.token, self.host,
+            self.port if self.port else (443 if self.secure else 80),
+        )
+
     def url(self, privacy=False, *args, **kwargs):
         """
         Returns the URL built dynamically based on specified arguments.
