@@ -50,9 +50,7 @@ from json import loads
 from .base import NotifyBase
 from ..url import PrivacyMode
 from ..common import NotifyType
-from ..utils import is_phone_no
-from ..utils import parse_phone_no
-from ..utils import validate_regex
+from ..utils.parse import is_phone_no, parse_phone_no, validate_regex
 from ..locale import gettext_lazy as _
 
 
@@ -413,6 +411,18 @@ class NotifyTwilio(NotifyBase):
 
         return not has_error
 
+    @property
+    def url_identifier(self):
+        """
+        Returns all of the identifiers that make this URL unique from
+        another simliar one. Targets or end points should never be identified
+        here.
+        """
+        return (
+            self.secure_protocol, self.account_sid, self.auth_token,
+            self.source,
+        )
+
     def url(self, privacy=False, *args, **kwargs):
         """
         Returns the URL built dynamically based on specified arguments.
