@@ -59,10 +59,8 @@ from .base import NotifyBase
 from ..common import NotifyImageSize
 from ..common import NotifyType
 from ..common import NotifyFormat
-from ..utils import parse_bool
-from ..utils import validate_regex
-from ..utils import apply_template
-from ..utils import TemplateType
+from ..utils.parse import parse_bool, validate_regex
+from ..utils.templates import apply_template, TemplateType
 from ..apprise_attachment import AppriseAttachment
 from ..locale import gettext_lazy as _
 
@@ -432,6 +430,18 @@ class NotifyWorkflows(NotifyBase):
 
         return True
 
+    @property
+    def url_identifier(self):
+        """
+        Returns all of the identifiers that make this URL unique from
+        another simliar one. Targets or end points should never be identified
+        here.
+        """
+        return (
+            self.secure_protocol[0], self.host, self.port, self.workflow,
+            self.signature,
+        )
+
     def url(self, privacy=False, *args, **kwargs):
         """
         Returns the URL built dynamically based on specified arguments.
