@@ -29,9 +29,11 @@
 from .rest import AppriseURLTester
 from .asyncio import OuterEventLoop
 from .module import reload_plugin
+from .environment import environ
 
 __all__ = [
     'AppriseURLTester',
     'OuterEventLoop',
     'reload_plugin',
+    'environ',
 ]
