@@ -52,7 +52,7 @@ from apprise.locale import LazyTranslation
 from apprise.locale import gettext_lazy as _
 
 from apprise import NotificationManager
-from apprise.utils import parse_list
+from apprise.utils.parse import parse_list
 from helpers import OuterEventLoop
 import inspect
 
