@@ -29,6 +29,8 @@
 import re
 from unittest import mock
 
+import pytest
+from apprise import exception
 import requests
 import mimetypes
 from os.path import join
@@ -86,15 +88,21 @@ def test_attach_http_query_string_dictionary():
 
     """
 
-    # no qsd specified
-    results = AttachHTTP.parse_url('http://localhost')
+    # Set verify off
+    results = AttachHTTP.parse_url('http://localhost?verify=no&rto=9&cto=8')
     assert isinstance(results, dict)
 
     # Create our object
     obj = AttachHTTP(**results)
     assert isinstance(obj, AttachHTTP)
 
-    assert re.search(r'[?&]verify=yes', obj.url())
+    # verify is disabled and therefore set
+    assert re.search(r'[?&]verify=no', obj.url())
+
+    # Our connect timeout flag is set since it differs from the default
+    assert re.search(r'[?&]cto=8', obj.url())
+    # Our read timeout flag is set since it differs from the default
+    assert re.search(r'[?&]rto=9', obj.url())
 
     # Now lets create a URL with a custom Query String entry
 
@@ -106,7 +114,8 @@ def test_attach_http_query_string_dictionary():
     obj = AttachHTTP(**results)
     assert isinstance(obj, AttachHTTP)
 
-    assert re.search(r'[?&]verify=yes', obj.url())
+    # verify is not in the URL as it is implied (default)
+    assert not re.search(r'[?&]verify=yes', obj.url())
 
     # But now test that our custom arguments have also been set
     assert re.search(r'[?&]dl=1', obj.url())
@@ -474,3 +483,33 @@ def test_attach_http(mock_get, mock_post):
     assert mock_post.call_count == 30
     # We only fetched once and re-used the same fetch for all posts
     assert mock_get.call_count == 1
+
+    #
+    # We will test our base64 handling now
+    #
+    mock_get.reset_mock()
+    mock_post.reset_mock()
+
+    AttachHTTP.max_file_size = getsize(path)
+    # Set ourselves a Content-Disposition (providing a filename)
+    dummy_response.headers['Content-Disposition'] = \
+        'attachment; filename="myimage.gif"'
+    results = AttachHTTP.parse_url('http://user@localhost/filename.gif')
+    assert isinstance(results, dict)
+    obj = AttachHTTP(**results)
+
+    # now test our base64 output
+    assert isinstance(obj.base64(), str)
+    # No encoding if we choose
+    assert isinstance(obj.base64(encoding=None), bytes)
+
+    # Error cases:
+    with mock.patch("builtins.open", new_callable=mock.mock_open,
+                    read_data="mocked file content") as mock_file:
+        mock_file.side_effect = FileNotFoundError
+        with pytest.raises(exception.AppriseFileNotFound):
+            obj.base64()
+
+        mock_file.side_effect = OSError
+        with pytest.raises(exception.AppriseDiskIOError):
+            obj.base64()
