@@ -28,9 +28,9 @@
 
 from unittest import mock
 
-from apprise.config.ConfigFile import ConfigFile
-from apprise.plugins.NotifyBase import NotifyBase
-from apprise.AppriseAsset import AppriseAsset
+from apprise.config.file import ConfigFile
+from apprise.plugins import NotifyBase
+from apprise import AppriseAsset
 
 # Disable logging for a cleaner testing output
 import logging
